=head1 NAME

package Mail::SpamAssassin::Plugin::reps

=head1 SYNOPSIS

  loadplugin Mail::SpamAssassin::Plugin::reps

=head1 DESCRIPTION

# reps
# Is a SpamAssassin Plugin that search into maillog for repeated subject received in near past
# Very useful to be mixed with other scores to detect mass spaming


=cut
package Mail::SpamAssassin::Plugin::reps;

use strict;
use warnings;
use bytes;
use Socket;
use Net::DNS;
use Mail::Address;
use Mail::SpamAssassin;
use Mail::SpamAssassin::PerMsgStatus;
use Mail::SpamAssassin::Logger;
use Mail::SpamAssassin::Plugin;
use Mail::SpamAssassin::Message::Metadata::Received qw(parse_received_line);

use vars qw(@ISA);
@ISA = qw(Mail::SpamAssassin::Plugin);


sub new {
  my ($class, $mailsa) = @_;
  $class = ref($class) || $class;
  my $self = $class->SUPER::new($mailsa);
  bless ($self, $class);
  #
  $self->set_config($mailsa->{conf});
  $self->register_eval_rule ( 'reps_same_sender' );
  $self->register_eval_rule ( 'reps_mixed_sender' );
  #
  #$self;
}


# set_config
# ==========
# Load configuration parameters
sub set_config {
  my ($self, $conf) = @_;
  my @cmds = ();

  push (@cmds, {
        setting => 'reps_sql_dsn',
        type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING,
  });

  push (@cmds, {
        setting => 'reps_sql_username',
        type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING,
  });

  push (@cmds, {
        setting => 'reps_sql_password',
        type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING,
  });

  push (@cmds, {
        setting => 'reps_subjectlen',
        type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING,
  });

  push (@cmds, {
        setting => 'reps_minutesback',
        type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING,
  });

  push (@cmds, {
        setting => 'reps_occurrences',
        type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING,
  });

  push (@cmds, {
        setting => 'reps_diffsenders',
        type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING,
  });

  $conf->{parser}->register_commands(\@cmds);
  #---dbg("reps: set_config: Configuration Loaded");
}



# reps_same_sender
# ----------------
# Check for repeated subject from same sender
sub reps_same_sender {
 #---dbg("reps: reps_same_sender: Start");
 my ($self, $params) = @_;
 my $msgr = $params->get_message();
 # Find ip and exit if external
 my $msgip = "";
 foreach my $rly  (@{$params->{relays_untrusted}}) {
  if ($rly->{ip}) {$msgip = $rly->{ip}; last;}
 }
 if ($msgip eq "") {
  dbg("reps: reps_same_sender: Skipped test due to internal ip ");
  return 0;
 }
 #---dbg("reps: reps_same_sender: Sender Ip: " . $msgip);
 # Get From Address
 my $msgfrom = lc $params->get( 'From:addr' );
 $msgfrom =~ s/^\s+|\s+$//g;
 #---dbg("reps: reps_same_sender: MsgFrom: " . $msgfrom);
 my $msgsubj = $params->get( 'Subject' );
 $msgsubj =~ s/^\s+|\s+$//g;
 #---dbg("reps: reps_same_sender: MsgSubj: " . $msgsubj);
 #---dbg("reps: reps_same_sender: Subject Len: " . length($msgsubj));
 # Get parameters
 my $main = $self->{main};
 my $subjectlen = int($main->{conf}->{reps_subjectlen});
 #---dbg("reps: reps_same_sender: MinimumSubjectLen: " . $subjectlen);
 if (length($msgsubj) < $subjectlen) {
  dbg("reps: reps_same_sender: Skipped test due to sublect lenght less than " . $subjectlen);
  return 0;
 }
 # Minutes to go back in search same subject
 my $minutesback = int($main->{conf}->{reps_minutesback});
 #---dbg("reps: reps_same_sender: MinutesBack: " . $minutesback);
 # Occurrences needed to hit in search same subject
 my $occurrences = int($main->{conf}->{reps_occurrences});
 #---dbg("reps: reps_same_sender: Occurrences: " . $occurrences);
 # Get db parameters
 my $dsn = $self->{main}->{conf}->{reps_sql_dsn};
 require DBI;
 my $dbuser = $main->{conf}->{reps_sql_username};
 my $dbpass = $main->{conf}->{reps_sql_password};
 # Connect DB
 my $dbh = DBI->connect($dsn, $dbuser, $dbpass, {'PrintError' => 0});
 if (!$dbh) { die("reps: reps_same_sender: different senders: " . $DBI::errstr); }
 # Search for a record key and set score if found
 my $sql = "SELECT COUNT(id) AS cc FROM maillog WHERE timestamp >= (now() - INTERVAL $minutesback minute) AND from_address = '$msgfrom' AND spamwhitelisted = 0 AND subject = ? ";
 #---dbg("reps: reps_same_sender: Sql: " . $sql);
 my $vcc = "";
 my $sth = $dbh->prepare($sql);
 #---dbg("reps: reps_same_sender: msgsubj: " . $msgsubj);
 $sth->execute($msgsubj);
 $vcc = $sth->fetchrow();
 $sth->finish();
 if ($vcc) {
  #---dbg("reps: reps_same_sender: count: " . $vcc);
  # Test if number of occurrences is enough to hit as in parameters
  if ($vcc >= $occurrences) {
   dbg("reps: reps_same_sender: GOT HIT (" . $vcc . ")");
   # Disconnect DB
   $dbh->disconnect();
   #---dbg("reps: reps_same_sender: Disconnected DB");
   return 1;
  }
 }
 dbg("reps: reps_same_sender: pass");

 # Disconnect DB
 $dbh->disconnect();
 #---dbg("reps: reps_same_sender: Disconnected DB");
 return 0;
}



# reps_mixed_sender
# ----------------
# Check for repeated subject from mixed sender
sub reps_mixed_sender {
 #---dbg("reps: reps_mixed_sender: Start");
 my ($self, $params) = @_;
 my $msgr = $params->get_message();
 # Find ip and exit if external
 my $msgip = "";
 foreach my $rly  (@{$params->{relays_untrusted}}) {
  if ($rly->{ip}) {$msgip = $rly->{ip}; last;}
 }
 if ($msgip eq "") {
  dbg("reps: reps_mixed_sender: Skipped test due to internal ip ");
  return 0;
 }
 #---dbg("reps: reps_mixed_sender: Sender Ip: " . $msgip);
 # Get From Address
 my $msgfrom = lc $params->get( 'From:addr' );
 $msgfrom =~ s/^\s+|\s+$//g;
 #---dbg("reps: reps_mixed_sender: MsgFrom: " . $msgfrom);
 my $msgsubj = $params->get( 'Subject' );
 $msgsubj =~ s/^\s+|\s+$//g;
 #---dbg("reps: reps_mixed_sender: MsgSubj: " . $msgsubj);
 #---dbg("reps: reps_mixed_sender: Subject Len: " . length($msgsubj));
 # Get parameters
 my $main = $self->{main};
 my $subjectlen = int($main->{conf}->{reps_subjectlen});
 #---dbg("reps: reps_mixed_sender: MinimumSubjectLen: " . $subjectlen);
 if (length($msgsubj) < $subjectlen) {
  dbg("reps: reps_mixed_sender: Skipped test due to sublect lenght less than " . $subjectlen);
  return 0;
 }
 # Minutes to go back in search same subject
 my $minutesback = int($main->{conf}->{reps_minutesback});
 #---dbg("reps: reps_mixed_sender: MinutesBack: " . $minutesback);
 # Different senders needed to hit in search same subject
 my $diffsenders = int($main->{conf}->{reps_diffsenders});
 #---dbg("reps: reps_mixed_sender: Diffsenders: " . $diffsenders);
 # Get db parameters
 my $dsn = $self->{main}->{conf}->{reps_sql_dsn};
 require DBI;
 my $dbuser = $main->{conf}->{reps_sql_username};
 my $dbpass = $main->{conf}->{reps_sql_password};
 # Connect DB
 my $dbh = DBI->connect($dsn, $dbuser, $dbpass, {'PrintError' => 0});
 if (!$dbh) { die("reps: reps_mixed_sender: different senders: " . $DBI::errstr); }
 # Search for a record key and set score if found
 my $sql = "SELECT from_address FROM maillog WHERE timestamp >= (now() - INTERVAL $minutesback minute)  AND spamwhitelisted = 0 AND subject = ? ORDER BY from_address";
 #---dbg("reps: reps_mixed_sender: Sql: " . $sql);
 my $vcc = "";
 my $sth = $dbh->prepare($sql);
 #---dbg("reps: reps_mixed_sender: msgsubj: " . $msgsubj);
 $sth->execute($msgsubj);
 if ($@) { dbg "reps: reps_mixed_sender: Something happened."; }
 my $msgcount = 0;
 my $senderprev = "";
 my $diffsendernum = 0;
 while ($vcc = $sth->fetchrow_array) { 
  #---dbg("reps: reps_mixed_sender: sender row: " . $vcc);
  $msgcount++; 
  if ($senderprev ne $vcc && $msgcount > 1 ) {
   $diffsendernum++;
   #---dbg("reps: reps_mixed_sender: different senders: " . $diffsendernum);
  }
  $senderprev = $vcc;
 }
 $sth->finish();
 #---dbg("reps: reps_mixed_sender: count: " . $msgcount);
 # Test if number of different senders is enough to hit as in parameters
 if ($diffsendernum >= $diffsenders) {
  dbg("reps: reps_mixed_sender: GOT HIT (" . $msgcount . ")");
  # Disconnect DB
  $dbh->disconnect();
  #---dbg("reps: reps_mixed_sender: Disconnected DB");
  return 1;
 }
 dbg("reps: reps_mixed_sender: pass");

 # Disconnect DB
 $dbh->disconnect();
 #---dbg("reps: reps_mixed_sender: Disconnected DB");
 return 0;
}


1;
